{
    Copyright (c) 2011 by Sven Barth

    Android application for managing train rides.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 ****************************************************************************
}

unit TrainData;

{$namespace org.sven.trainlog}
{$mode objfpc}{$H+}
{$modeswitch unicodestrings}

interface

uses
  AndroidR14;

type
  ETrainLogData = class(JLException)
  end;
  ETrainLog = class(JLException)
  end;

  TTrainLogHelper = class(ADSSqliteOpenHelper)
  strict private
    function GetNameOfTrainFamily(aID: jlong): String;
    procedure SetNameOfTrainFamily(aID: jlong; aValue: String);
    function GetNameOfTrainType(aID: jlong): String;
    procedure SetNameOfTrainType(aID: jlong; aValue: String);
    function GetNameOfTrain(aID: jlong): String;
    procedure SetNameOfTrain(aID: jlong; aValue: String);
    function GetCountOfTrain(aID: jlong): jint;
    procedure SetCountOfTrain(aID: jlong; aValue: jint);
  private
    const
      DatabaseVersion = 1;
      DatabaseName = 'trainlog';
      CreateTableTrainFamily = 'create table TrainFamily (' +
                               '  _id integer primary key autoincrement,' +
                               '  name varchar not null' +
                               ')';
      CreateTableTrainType = 'create table TrainType (' +
                             '  _id integer primary key autoincrement,' +
                             '  name varchar not null,' +
                             '  family_id integer not null,' +
                             '  foreign key(family_id) references ' +
                             '    TrainFamily(_id) on delete cascade' +
                             ')';
      CreateTableTrain = 'create table Train (' +
                         '  _id integer primary key autoincrement,' +
                         '  name varchar not null,' +
                         '  cnt integer not null default 0,' +
                         '  type_id integer not null,' +
                         '  foreign key(type_id) references TrainType(_id)' +
                         '    on delete cascade' +
                         ')';
  public
    const
      ColumnTrainFamilyName = 'name';
      ColumnTrainTypeName = 'name';
      ColumnTrainTypeFamilyID = 'family_id';
      ColumnTrainName = 'name';
      ColumnTrainCount = 'cnt';
      ColumnTrainTypeID = 'type_id';

      TableTrainFamily = 'TrainFamily';
      TableTrainType = 'TrainType';
      TableTrain = 'Train';
  public
    constructor Create(aContext: ACContext);
    procedure onCreate(aDatabase: ADSSqliteDatabase); override;
    procedure onUpgrade(aDatabase: ADSSqliteDatabase;
      aOldVersion, aNewVersion: jint); override;
    procedure onOpen(aDatabase: ADSSqliteDatabase); override;

    function GetAllTrainFamilies: ADCursor;
    function GetTrainTypesByFamily(aFamily: jlong): ADCursor;
    function GetTrainsByType(aType: jlong): ADCursor;

    procedure AddTrainFamily(aName: String);
    procedure AddTrainType(aName: String; aFamily: jlong);
    procedure AddTrain(aName: String; aType: jlong);

    procedure DeleteTrainFamily(aID: jlong);
    procedure DeleteTrainType(aID: jlong);
    procedure DeleteTrain(aID: jlong);
  public
    property NameOfTrainFamily[aID: jlong]: String read GetNameOfTrainFamily
      write SetNameOfTrainFamily;
    property NameOfTrainType[aID: jlong]: String read GetNameOfTrainType
      write SetNameOfTrainType;
    property NameOfTrain[aID: jlong]: String read GetNameOfTrain
      write SetNameOfTrain;
    property CountOfTrain[aID: jlong]: jint read GetCountOfTrain
      write SetCountOfTrain;
  end;

implementation

{ TTrainLogHelper }

constructor TTrainLogHelper.Create(aContext: ACContext);
begin
  inherited Create(aContext, DatabaseName, Nil, DatabaseVersion);
end;

procedure TTrainLogHelper.onCreate(aDatabase: ADSSqliteDatabase);
begin
  aDatabase.execSQL(CreateTableTrainFamily);
  aDatabase.execSQL(CreateTableTrainType);
  aDatabase.execSQL(CreateTableTrain);
end;

procedure TTrainLogHelper.onUpgrade(aDatabase: ADSSqliteDatabase;
  aOldVersion, aNewVersion: jint);
begin
  // nothing
end;

procedure TTrainLogHelper.onOpen(aDatabase: ADSSqliteDatabase);
begin
  inherited onOpen(aDatabase);
  if not aDatabase.isReadOnly then
    aDatabase.ExecSQL('pragma foreign_keys=on;');
end;

function TTrainLogHelper.GetAllTrainFamilies: ADCursor;
begin
  Result := getReadableDatabase.rawQuery(
              'select * from TrainFamily order by name asc', Nil);
end;

function TTrainLogHelper.GetTrainTypesByFamily(aFamily: jlong): ADCursor;
var
  query: String;
begin
  query := JLLong.Create(aFamily).toString;
  query := 'select * from TrainType where family_id = ' + query + ' order by ' +
           'name asc';
  Result := getReadableDatabase.rawQuery(query, Nil);
end;

function TTrainLogHelper.GetTrainsByType(aType: jlong): ADCursor;
var
  query: String;
begin
  query := JLLong.Create(aType).toString;
  query := 'select * from Train where type_id = ' + query +
           ' order by name asc';
  Result := getReadableDatabase.rawQuery(query, Nil);
end;

procedure TTrainLogHelper.AddTrainFamily(aName: String);
var
  values: ACContentValues;
begin
  values := ACContentValues.Create(1);
  values.put(ColumnTrainFamilyName, aName);
  getWritableDatabase.insert(TableTrainFamily, Nil, values);
end;

procedure TTrainLogHelper.AddTrainType(aName: String; aFamily: jlong);
var
  values: ACContentValues;
begin
  values := ACContentValues.Create(2);
  values.put(ColumnTrainTypeName, aName);
  values.put(ColumnTrainTypeFamilyID, JLLong.Create(aFamily));
  getWritableDatabase.insert(TableTrainType, Nil, values);
end;

procedure TTrainLogHelper.AddTrain(aName: String; aType: jlong);
var
  values: ACContentValues;
begin
  values := ACContentValues.Create(2);
  values.put(ColumnTrainName, aName);
  values.put(ColumnTrainTypeID, JLLong.Create(aType));
  getWritableDatabase.insert(TableTrain, Nil, values);
end;

function TTrainLogHelper.GetNameOfTrainFamily(aID: jlong): String;
var
  cur: ADCursor;
  id: String;
  query: JLString;
begin
  id := JLLong.Create(aID).toString;
  cur := getReadableDatabase.rawQuery(
           'select name from TrainFamily where _id = ' + id, Nil);
  if cur.getCount = 0 then
    raise ETrainLogData.Create('TrainFamily not found');
  cur.moveToFirst;
  Result := cur.getString(0);
end;

procedure TTrainLogHelper.SetNameOfTrainFamily(aID: jlong; aValue: String);
var
  where: String;
  values: ACContentValues;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  values := ACContentValues.create;
  values.put(ColumnTrainFamilyName, aValue);
  getWritableDatabase.update(TableTrainFamily, values, where, Nil);
end;

function TTrainLogHelper.GetNameOfTrainType(aID: jlong): String;
var
  cur: ADCursor;
  id: String;
  query: JLString;
begin
  id := JLLong.Create(aID).toString;
  cur := getReadableDatabase.rawQuery(
           'select name from TrainType where _id = ' + id, Nil);
  if cur.getCount = 0 then
    raise ETrainLogData.Create('TrainType not found');
  cur.moveToFirst;
  Result := cur.getString(0);
end;

procedure TTrainLogHelper.SetNameOfTrainType(aID: jlong; aValue: String);
var
  where: String;
  values: ACContentValues;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  values := ACContentValues.create;
  values.put(ColumnTrainTypeName, aValue);
  getWritableDatabase.update(TableTrainType, values, where, Nil);
end;

function TTrainLogHelper.GetNameOfTrain(aID: jlong): String;
var
  cur: ADCursor;
  id: String;
  query: JLString;
begin
  id := JLLong.Create(aID).toString;
  cur := getReadableDatabase.rawQuery(
           'select name from Train where _id = ' + id, Nil);
  if cur.getCount = 0 then
    raise ETrainLogData.Create('Train not found');
  cur.moveToFirst;
  Result := cur.getString(0);
end;

procedure TTrainLogHelper.SetNameOfTrain(aID: jlong; aValue: String);
var
  where: String;
  values: ACContentValues;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  values := ACContentValues.create;
  values.put(ColumnTrainName, aValue);
  getWritableDatabase.update(TableTrain, values, where, Nil);
end;

function TTrainLogHelper.GetCountOfTrain(aID: jlong): jint;
var
  cur: ADCursor;
  id: String;
  query: JLString;
begin
  id := JLLong.Create(aID).toString;
  cur := getReadableDatabase.rawQuery(
           'select cnt from Train where _id = ' + id, Nil);
  if cur.getCount = 0 then
    raise ETrainLogData.Create('Train not found');
  cur.moveToFirst;
  Result := cur.getInt(0);
end;

procedure TTrainLogHelper.SetCountOfTrain(aID: jlong; aValue: jint);
var
  where: String;
  values: ACContentValues;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  values := ACContentValues.create;
  values.put(ColumnTrainCount, JLInteger.Create(aValue));
  getWritableDatabase.update(TableTrain, values, where, Nil);
end;

procedure TTrainLogHelper.DeleteTrainFamily(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  getWritableDatabase.delete(TableTrainFamily, where, Nil);
end;

procedure TTrainLogHelper.DeleteTrainType(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  getWritableDatabase.delete(TableTrainType, where, Nil);
end;

procedure TTrainLogHelper.DeleteTrain(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  getWritableDatabase.delete(TableTrain, where, Nil);
end;

end.

